import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="packel",
    version="1.0.0",
    author="kiwec",
    author_email="c.wolf@kiwec.net",
    description="Packet serialization/deserialization in a Pythonic way.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kiwec/packel",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Public Domain",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
